/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2022  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QApplication>
#include <QStyleFactory>
#include <QScreen>
#include <QFont>
#include <QSize>

#ifdef Q_OS_MAC
#include "../../core/nspool.h"
#endif

#include "vibrationmeter.h"

int main(int argc, char *argv[])
{
	int									ret_value;
    QScreen                             *screen;
    double                              dpi;
    QSize                               size;
    int                                 line_height;
    int                                 font_pnt_size(2);
    static const int                    MIN_LINES(50.0);

    QApplication::setAttribute(Qt::AA_Use96Dpi);

#ifdef Q_OS_MAC
	TNSPool *nspool = new TNSPool();
#endif
	QApplication app(argc, argv);

    screen = QApplication::primaryScreen();

    if(screen)
    {
        dpi = screen->logicalDotsPerInchY();
        size = screen->size();

        line_height = (size.height()) / MIN_LINES;
        font_pnt_size = static_cast<int>(static_cast<double>(line_height + 1) * (72.0 / dpi));
		
		if(font_pnt_size > 12) font_pnt_size = 12;
		
		qWarning(QString("font_pnt_size: %1").arg(font_pnt_size).toLatin1());
    }

    if(font_pnt_size < 2)
    {
        font_pnt_size = 2;
    }

    app.setFont(QFont("Helvetica",font_pnt_size));
	
#ifndef Q_OS_MAC
	QApplication::setStyle(QStyleFactory::create("fusion"));
#endif
	
	TVibrationMeter *w = new TVibrationMeter;
	w->show();
	
#ifdef Q_OS_ANDROID
	w->setWindowState(Qt::WindowFullScreen);
#endif
	
	ret_value = app.exec();
	
#ifdef Q_OS_MAC
	delete nspool;
#endif
	
	delete w;
	return ret_value;
}
